/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Appkit/Appkit.h>
#import "IDLParser.h"

@implementation IDLParser(Viewer)

- (void) _loadInterfacesWithTypes:(NSArray*) types window:window
{
    NSOpenPanel* openPanel = [NSOpenPanel openPanel];
	
    [openPanel setTitle:@"Open IDL files"];
    [openPanel setAllowsMultipleSelection:YES];
    [openPanel setDelegate:self];
    [openPanel setCanChooseDirectories:YES];
    [openPanel setCanChooseFiles:YES];
	
    [openPanel beginSheetForDirectory:nil file:nil types:nil
        modalForWindow:window modalDelegate:self 
        didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:) contextInfo:NULL];
}
    
- (void)openPanelDidEnd:(NSOpenPanel *)openPanel 
        returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if(returnCode == NSOKButton){
        NSArray* filenames = [openPanel filenames];
        NSString* file;
        NSEnumerator* en = [filenames objectEnumerator];
        NSMutableArray* array = [NSMutableArray array];
        NSFileManager* fm = [NSFileManager defaultManager];
		[openPanel orderOut:self];
        while(file = [en nextObject]){
            NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
            BOOL isDir;
            BOOL ok = [fm fileExistsAtPath:file isDirectory:&isDir];
            if(ok && !isDir && [[file pathExtension]isEqualToString:@"idl"])
                [array addObject:file];
            else
            if(ok && isDir){
                NSEnumerator* dirEn = [fm enumeratorAtPath:file];
                NSString* path;
                [self addSearchPath:file];
                while(path = [dirEn nextObject]){
                    path = [file stringByAppendingPathComponent:path];
                    ok = [fm fileExistsAtPath:path isDirectory:&isDir];
                    if(ok && !isDir && [[path pathExtension]isEqualToString:@"idl"])
                        [array addObject:path];
                }
            }
            [pool release];
        }
        
        [self loadFiles:array];
    }
}

- (void) loadInterfacesWithTypes:(NSArray*)types window:window
{
    if(!_root)
        [self _loadInterfacesWithTypes:types window: window];
    else
        NSBeep();
}

@end
