/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLExceptionController.h"
#import "IDLException.h"
#import "IDLItem.h"

@implementation IDLExceptionController

- (NSString*) nibName
{
	return @"IDLExceptionController";
}

- (BOOL) isEditable
{
	return NO;
}

- initWithException:(NSException*)excp definition:(IDLException*)def
{
	self = [super initWithDefinition:def];
	_exception = [excp retain];
	[_item setValue:[_exception userInfo]];
	return self;
}

- (void) dealloc
{
	[_exception release];
	[super dealloc];
}

- (void) ok:sender
{
	if(_item)
		[NSApp endSheet:window];
}

- (NSException*) exception
{
	return _exception;
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:runner
{
	[window orderOut:self];
	[window autorelease];
	[self autorelease];
}

- (void) showExceptionFor:runner window:docWindow
{
    if(_item){
        [NSApp beginSheet:window modalForWindow:docWindow 
            modalDelegate:self 
            didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
            contextInfo:self];
    }else{ // standard NSException
        NSDictionary* info = [_exception userInfo];
        NSBeginAlertSheet([_exception name], nil,nil,nil,
            docWindow, self, 
            @selector(sheetDidEnd:returnCode:contextInfo:), NULL,
            runner, 
            @"%@\n%@",[_exception reason], info? [info description] : @"");
    }
}

@end
