/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>

@class IDLParser;
@class IORView;

@interface IDLViewer:NSObject
{
	IBOutlet NSWindow* window;
	IBOutlet NSBrowser* typesBrowser;
	IBOutlet NSTextField* typeField;
	IBOutlet IORView* objectView;
	IBOutlet NSOutlineView* table;
	IBOutlet NSDrawer* drawer;
	IBOutlet NSTextField* messageField;
	IBOutlet NSArrayController* fileController;
	IDLParser* parser;
	id _dict;
	id _cache;
	id _objects;
	struct {
		unsigned allFiles:1;
		unsigned:31;
	} _flags;
}

- initWithParser:(IDLParser*)parser;
- window;
- (void) orderFront:sender;
- (void) reload:sender;
- (void) click:sender;
- (void) saveDocument:sender;
- (void) saveDocumentAs:sender;

- selectedObject;

- (void) getTemplate:sender;
- (void) loadInterface:sender;
- (void) parse:sender;

- (NSArray*)modules;

@end

@interface IDLViewer(Logging)
- (void) notifyParseError:(NSNotification*)notification;

@end

extern NSString* IDLDefinitionPboardType;

@interface IDLDefinition(Viewer)

- (void) loadCell:cell;
- (void) fillMatrix:matrix;
- (NSImage*) cellImage;
- (NSString*) category;
- (NSString*) title;
- (NSString*) stringValue;

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard;

@end

@interface NSObject(LoadCell)
- (void) fillMatrix:matrix;
- (void) loadCell:cell;
@end

@interface IDLDefinitionScope(Viewer)

- (NSString*) title;
- (NSString*)category;
- (void) fillMatrix:matrix;
- (NSArray*) modules;

@end

