/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <ADORB/ADORB.h>
#import "DemoServer.h"
#import "NamingContextProto.h"
#import "ServerProto.h"

@implementation DemoServer

static NSString* interfaceName = @"demo/Server";

- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:interfaceName];
}

- (void) run
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	@try{
		[CORBAObject setDefaultConnectTimeout:10.0];
		CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
		// If NameService is not specified try to connect
		if(!nameService){
			// Get ORBInitialPort of the naming service, if it is passed as -port ...
			NSString* nsPort = [defaults stringForKey:@"port"];
			// If not specified, assume 15600. The default orbd's ORBInitialPort is different.
			if(!nsPort)  
				nsPort = @"15600"; //
			// Form corbaloc URL
			NSString* nsURL = [NSString stringWithFormat:@"corbaloc:iiop:1.2@localhost:%@", nsPort];
			// Get name service
			nameService = [CORBAObject objectWithURL:nsURL];
			// corbaloc provides no information of what interface 'ns' implements
			// so we need to set interface explicitly
			// Get interface object for NamingContext and set it
			IDLInterface* nsInterface = [[ORB defaultORB]interfaceNamed:@"CosNaming/NamingContext"];
			[nameService setInterface:nsInterface];
		}
		// Bind self in Naming Service, if provided
		// We don't need to activate 'self' explicitly since it will be auto-activated
		if(nameService){
			CORBAObject* ctxt = nil;
			@try{
				// Create new context in the Naming Service with the name 'demo'
				ctxt = [nameService bindNewContext:[IDLNameComponent nameComponentsForPath:@"demo"]];
			}@catch(NSException *localException){
				BOOL ignore = NO;
				if([[localException name]isEqualToString:CORBAUserException]){
					NSString* exceptionName = [[localException userInfo]valueForKey:@"name"];
					if([exceptionName isEqualToString:@"AlreadyBound"])
						ignore = YES;
				}
				if(!ignore){
					NSLog(@"Unable to bind new context, %@, %@, userInfo %@",
						[localException name], [localException reason], [localException userInfo]);
					[localException raise];
				}
			}
			// Bind self in Naming Context. The reference wiil be bound in created context
			[nameService rebind:[IDLNameComponent nameComponentsForPath:@"demo/DemoServer"] object:self];
		}else{
			// If no 'nameService' we have to activate 'self' explicitly to make it available
			[self activate];
			// Save IOR to the file
			NSString* ior = [[self objectReference]toString];
			NSString* path = [[NSFileManager defaultManager]currentDirectoryPath];
			if(![ior writeToFile:[path stringByAppendingPathComponent:@"DemoServer.ref"] atomically:NO encoding:NSUTF8StringEncoding error:NULL])
				NSLog(@"Cannot write IOR to file './DemoServer.ref'\n");
			else
				NSLog(@"%@: Saved IOR in './DemoServer.ref'\n", [self className]);
		}
	}@catch(NSException *localException){
		NSLog(@"%@, %@", [localException name], [localException reason]);
	}
	// Run ORB
	NSLog(@"DemoServer: ORB is running ...\n");
	[ORB run];
}

- (NSArray*) echoPath:(NSArray*)pathArray
{
	return pathArray;
}

- echoNode:node
{
	return node;
}

- getAccessFor:client
{
//	NSString* name = @"dummy";
	NSString* name = [client getName]; // call back client
	// we do not need to set IDL type since returned type is struct
	IDLObject* object = [IDLObject objectWithIDLType:nil];
	[object setValue:name forKey:@"name"];
	[object setValue:self forKey:@"object"];
	return object;
}

- echoAny:object
{
	return object;
}

@end
