/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLAny.h"
#import "IDLDefinition_Private.h"
#import "IDLAnyObject.h"

@implementation IDLAny

static IDLAny* _instance = nil;
static Class _defaultObjectClass = nil;

+ (void) initialize
{
	if([IDLAny class] == self){
		_defaultObjectClass = [IDLAnyObject class];
	}
}

+ (Class) defaultObjectClass
{
	return _defaultObjectClass;
}

+ (void) setDefaultObjectClass:(Class)class
{
	_defaultObjectClass = class;
}

+ any
{
    if(!_instance)
        _instance = [[self allocWithZone:[self zone]]init];
    return _instance;
}

- (NSString*) name
{
    return @"any";
}

- (long) kind
{
    return tk_any;
}

- (void) setObjectClassName:(NSString*)string
{
    [_className autorelease];
    _className = [string retain];
}

- (NSString*) objectClassName
{
    return _className;
}

- (void) encodeObject:obj withCoder:(id <CORBACoder>)coder
{
	if([obj isKindOfClass:[IDLAnyObject class]]){
		IDLDefinition* def = (IDLDefinition*)[obj idlType];
		if(def){
			[def encodeWithCORBACoder:coder];
			[def encodeObject:[obj value] withCoder:coder];
		}else
			[coder encodeLong:tk_null];
	}else{
		// Deprecated
		IDLDefinition* def = [obj valueForKey:@"type"];
		id value = [obj valueForKey:@"value"];
		if(def){
			[def encodeWithCORBACoder:coder];
			[def encodeObject:value withCoder:coder];
		}else//tk_null
			[coder encodeLong:tk_null];
    }
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    IDLDefinition* type = [IDLDefinition definitionFromCoder:coder];
    IDLDefinition* def = [type referenceType];
    if(def){
        id class = nil;
        if(_className)
            class = NSClassFromString(_className);
        if(!class)
            class = [[self class] defaultObjectClass];
		id value = nil;
		[def decodeObject:&value withCoder:coder];
		if([class instancesRespondToSelector:@selector(initWithIDLType:)])
			*obj = [[[class alloc]initWithIDLType:type]autorelease];
		else{
			*obj = [[[class alloc]init]autorelease];
			if([*obj respondsToSelector:@selector(setIDLType:)])
				[*obj setIDLType:type];
			else
				[*obj setValue:type forKey:@"type"];
		}
		[*obj setValue:value forKey:@"value"];
    }else
        *obj = nil; // if tk_null ==> def = nil
}

- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [coder encodeLong:[self kind]];
}


- (const char*) objCType
{
    return @encode(id);
}
- (unsigned) argLength
{
    return sizeof(id);
}
- (unsigned) returnValueLength
{
    return sizeof(id);
}

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    // Any is mapped to NSDictionary
    id dict = nil;
    [inv getArgument:&dict atIndex:n];
    return dict;
}
- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
    id dict = *((id*)ptr);
    return dict;
}

- (void) setValue:value ofType:(const char*)type forPointer:(void*)ptr
{
    *((id*)ptr) = value;
}
- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    id _val = value;
    [inv setArgument:&_val atIndex:n];
}
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    id _val = value;
    [inv setReturnValue:&_val];
}

- returnValueFromInvocation:(NSInvocation*)inv
{
    id _val = nil;
    [inv getReturnValue:&_val];
    return _val;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_className forKey:@"className"];
	}else{
		[coder encodeObject:_className];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_className = [[coder decodeObjectForKey:@"className"]retain];
	}else{
		_className = [[coder decodeObject]retain];
	}
	return self;
}

- (id)awakeAfterUsingCoder:(NSCoder *)aDecoder
{
	if(_instance){
		[self release];
		return _instance;
	}else
		return _instance = self;
}

@end
