/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinition_Private.h"
#import "IDLArray.h"
#import "IDLArrayIdentifier.h"
#import "IDLOctetArray.h"
#import "IDLEncapsulation.h"
#import "IDLLink.h"

@implementation IDLArray

- initWithType:(id<IDLDefinition>)aType identifier:(IDLArrayIdentifier*)ident
{
    self = [super init];
    _size = [ident arraySize];
    if([ident isNested])
        _type = [[IDLArray arrayWithType:aType identifier:[ident identifier]]retain];
    else
        _type =  [(NSObject*)aType retain];
    return self;
}

+ arrayWithType:(id<IDLDefinition>)aType identifier:(IDLArrayIdentifier*)ident
{
    id arrayClass = [(IDLDefinition*)aType arrayClass];
    return [[[arrayClass alloc]initWithType:aType identifier:ident]autorelease];
}

- (void) dealloc
{
    [_type release];
    [super dealloc];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeInt:_size forKey:@"size"];
	}else{
		[coder encodeObject:_type];
		[coder encodeValueOfObjCType:@encode(int) at:&_size];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_size = [coder decodeIntForKey:@"size"];
	}else{
		_type = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(int) at:&_size];
	}
	return self;
}
- (id)awakeAfterUsingCoder:(NSCoder *)aDecoder
{
	return self;
}

- (IDLDefinition*)arrayType
{
    return _type;
}

- (unsigned) arraySize
{
    return _size;
}

- (NSString*) name
{
    return [NSString stringWithFormat:@"%@[%d]", [[self arrayType]name],_size];
}

- (NSString*) description
{
    return [[_type description]stringByAppendingFormat:@"[%d]",_size];
}

- (long) kind
{
    return tk_array;
}

- _initWithType:(id<IDLDefinition>)aType size:(unsigned long)size
{
    self = [super init];
    _type = [(NSObject*)aType retain];
    _size = size;
    return self;
}


+ _definitionFromCoder:(IDLEncapsulation*)coder
{
    unsigned long size = 0;
    IDLDefinition* type;
    IDLLink* link = [IDLLink linkWithDefinition:nil];
    
    [coder registerDefinition:(IDLDefinition*)link forLocation:-2*sizeof(long)];

    type = [IDLDefinition definitionFromCoder: coder];
    [coder decodeULong:&size];
    if([type isKindOfClass:[IDLOctetArray class]]){
        IDLOctetArray* array = [[[IDLOctetArray allocWithZone:[self zone]]
                    _initWithType:[(IDLOctetArray*)type arrayType] 
                    size:size*[(IDLOctetArray*)type arraySize]]autorelease];
        [link setLink:array];
    }else{
        id arrayClass = [type arrayClass];
        IDLDefinition* array = [[[arrayClass allocWithZone:[self zone]]
                    _initWithType:type size:size]autorelease];
        [link setLink:array];
    }
    return link;
}

@end
