/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLStruct.h"
#import "IDLStructMember.h"

@implementation IDLStruct

+ (void) initialize
{
    if([IDLStruct class] == self){
        NSString* debug = [[NSUserDefaults standardUserDefaults]stringForKey:@"debug"];
        if(debug){
            NSRange r = [debug rangeOfString:@"V"];
            if(r.length > 0)
                [self setLogEnabled:YES];
        }
    }
}

- initWithName:(NSString*)name members:(NSArray*)members;
{
    self = [super init];
    structName = [name copy];
	if(members)
		_members = [[NSArray alloc]initWithArray:members];
    return self;
}

+ structWithName:(NSString*)aName members:(NSArray*)members;
{
    return [[[self allocWithZone:[self zone]]initWithName:aName members:members]autorelease];
}

- (void) dealloc
{
    [_members release];
    [structName release];
    [_className release];
    [super dealloc];
}


- (NSString*) name
{
    return structName;
}

- members
{
    return _members;
}

- (BOOL) setMembers:(NSArray*)members
{
    if(!_members && members){
        _members = [[NSArray alloc]initWithArray:members];
        return YES;
    }else
        return NO;
}

- memberWithName:(NSString*)name
{
	NSEnumerator* en = [_members objectEnumerator];
	IDLStructMember* member;
	while(member = [en nextObject])
		if([[member name]isEqualToString:name])
			return member;
	return nil;
}

- valueForKey:(NSString*)key
{
	if([key characterAtIndex:0] == '@')
		return [super valueForKey:[key substringFromIndex:1]];
	else
		return [self memberWithName:key];
}

- (void) setObjectClassName:(NSString*)string
{
    if(string != _className){
        [_className release];
        _className = [string copy];
    }
}
- (NSString*) objectClassName
{
    return _className;
}

- (NSString*) description
{
    return [[super description] stringByAppendingFormat:@"struct %@",[self name]];
}

- (long) kind
{
    return tk_struct;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:structName forKey:@"structName"];
		[coder encodeObject:_members forKey:@"members"];
		[coder encodeObject:_className forKey:@"className"];
	}else{
		[coder encodeObject:structName];
		[coder encodeObject:_members];
		[coder encodeObject:_className];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		structName = [[coder decodeObjectForKey:@"structName"]retain];
		_members = [[coder decodeObjectForKey:@"members"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
	}else{
		structName = [[coder decodeObject]retain];
		_members = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
	}
	return self;
}

@end
