/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLNameComponent.h"

@implementation IDLNameComponent

- init
{
    return [self initWithName:nil];
}
- initWithName:(NSString*)name
{
    self = [super init];
    if(name){
        NSRange r = [name rangeOfString:@"." options:NSBackwardsSearch];
        if(r.length > 0){
            _id = [[name substringToIndex:r.location]retain];
            _kind = [[name substringFromIndex:r.location+r.length]retain];
        }else
            _id = [name retain];
    }
    return self;
}
- (void) dealloc
{
    [_id release];
    [_kind release];
    [super dealloc];
}

- (NSString*)name
{
    if(_kind)
        return [NSString stringWithFormat:@"%@.%@",_id,_kind];
    else
        return _id;
}

+ nameComponentWithName:(NSString*)name
{
    return [[[self alloc]initWithName:name]autorelease];
}
+ (NSArray*) nameComponentsForPath:(NSString*)path
{
    NSArray* names = [path componentsSeparatedByString:@"/"];
    NSMutableArray* array = [NSMutableArray array];
    NSEnumerator* en = [names objectEnumerator];
    NSString* name;
    while(name = [en nextObject]){
        if([name length] > 0) // For the initial '/' and doubles '//'
            [array addObject:[self nameComponentWithName:name]];
    }
    return array;
}

- (NSString*) getId
{
    return _id;
}
- (void) _setId:(NSString*)str
{
    [_id autorelease];
    _id = [str retain];
}
- (void) setId:(NSString*)str
{
    [self _setId:str];
}
- (NSString*)getKind
{
    return _kind;
}
- (void) _setKind:(NSString*)str
{
    [_kind autorelease];
    _kind = [str retain];
}
- (void) setKind:(NSString*)str
{
    [self _setKind:str];
}

@end
